@@ -3,10 +3,12 @@
 import argparse
 import logging
 import os
+import subprocess
 import shlex
 import sys
 from decimal import Decimal
 from pathlib import Path
+from platform import platform
 from tempfile import NamedTemporaryFile
 
 try:
@@ -24,7 +26,7 @@ import argcomplete
 import colorama
 from appdirs import user_config_dir
 
-from .ansi import ANSIEmojiLoglevelFormatter, ANSIStrippingFormatter, ansi_colors, format_ansi
+from .ansi import ANSIEmojiLoglevelFormatter, ANSIStrippingEmojiLoglevelFormatter, ANSIStrippingFormatter, ansi_colors, ansi_escape, format_ansi
 from .configuration import Configuration, SubparserWrapper, get_argument_name, handle_store_boolean
 from .attrdict import AttrDict
 
@@ -54,11 +56,12 @@ class MILC(object):
         self.config_file = None
         self.default_arguments = {}
         self.version = 'unknown'
-        self.release_lock()
+        self.platform = platform()
 
         # Figure out our program name
         self.prog_name = sys.argv[0][:-3] if sys.argv[0].endswith('.py') else sys.argv[0]
         self.prog_name = os.environ.get('MILC_APP_NAME', os.path.basename(self.prog_name))
+        self.release_lock()
 
         # Initialize all the things
         self.initialize_config()
@@ -88,8 +91,60 @@ class MILC(object):
         args = args or kwargs
         text = format_ansi(text)
 
+        if not self.config.general.color:
+            text = ansi_escape.sub('', text)
+
         print(text % args)
 
+    def run(self, command, capture_output=True, combined_output=False, text=True, **kwargs):
+        """Run a command using `subprocess.run`, but using some different defaults.
+
+        Unlike subprocess.run you must supply a sequence of arguments. You can use `shlex.split()` to build this from a string.
+
+        The **kwargs arguments get passed directly to `subprocess.run`.
+
+        Args:
+            command
+                A sequence where the first item is the command to run, and any remaining items are arguments to pass.
+
+            capture_output
+                Set to False to have output written to the terminal instead of being available in the returned `subprocess.CompletedProcess` instance.
+
+            combined_output
+                When true STDERR will be written to STDOUT. Equivalent to the shell construct `2>&1`.
+
+            text
+                Set to False to disable encoding and get `bytes()` from `.stdout` and `.stderr`.
+        """
+        # Sanity Checking
+        if isinstance(command, str):
+            raise TypeError('`command` must be a non-text sequence such as list or tuple.')
+
+        if not capture_output and combined_output:
+            raise ValueError("Can't use capture_output=False and combined_output=True at the same time.")
+
+        # On some windows platforms (msys2, possibly others) you have to execute the command through a subshell.
+        if 'windows' in self.platform.lower():
+            safecmd = map(shlex.quote, command)
+            safecmd = ' '.join(safecmd)
+            command = [os.environ['SHELL'], '-c', safecmd]
+
+        # Argument Processing
+        if capture_output:
+            kwargs['stdout'] = subprocess.PIPE
+            kwargs['stderr'] = subprocess.PIPE
+
+        if combined_output:
+            kwargs['stderr'] = subprocess.STDOUT
+
+        if text:
+            kwargs['universal_newlines'] = True
+
+        # Run the command
+        self.log.debug('Running command: %s', command)
+
+        return subprocess.run(command, **kwargs)
+
     def initialize_argparse(self):
         """Prepare to process arguments from sys.argv.
         """
@@ -105,10 +160,24 @@ class MILC(object):
         self.args = AttrDict()
         self._arg_parser = argparse.ArgumentParser(**kwargs)
         self.set_defaults = self._arg_parser.set_defaults
-        self.print_usage = self._arg_parser.print_usage
-        self.print_help = self._arg_parser.print_help
         self.release_lock()
 
+    def print_help(self, *args, **kwargs):
+        """Print a help message for the main program or subcommand, depending on context.
+        """
+        if self._entrypoint.__name__ in self.subcommands:
+            return self.subcommands[self._entrypoint.__name__].print_help(*args, **kwargs)
+
+        return self._arg_parser.print_help(*args, **kwargs)
+
+    def print_usage(self, *args, **kwargs):
+        """Print brief description of how the main program or subcommand is invoked, depending on context.
+        """
+        if self._entrypoint.__name__ in self.subcommands:
+            return self.subcommands[self._entrypoint.__name__].print_usage(*args, **kwargs)
+
+        return self._arg_parser.print_usage(*args, **kwargs)
+
     def completer(self, completer):
         """Add an argcomplete completer to this subcommand.
         """
@@ -139,7 +208,7 @@ class MILC(object):
         self.log_print = True
         self.log_print_to = sys.stderr
         self.log_print_level = logging.INFO
-        self.log_file_level = logging.DEBUG
+        self.log_file_level = logging.INFO
         self.log_level = logging.INFO
         self.log = logging.getLogger(self.__class__.__name__)
         self.log.setLevel(logging.DEBUG)
@@ -151,6 +220,7 @@ class MILC(object):
         self.add_argument('--datetime-fmt', default='%Y-%m-%d %H:%M:%S', help='Format string for datetimes')
         self.add_argument('--log-fmt', default='%(levelname)s %(message)s', help='Format string for printed log output')
         self.add_argument('--log-file-fmt', default='[%(levelname)s] [%(asctime)s] [file:%(pathname)s] [line:%(lineno)d] %(message)s', help='Format string for log file.')
+        self.add_argument('--log-file-level', default='info', choices=['debug', 'info', 'warning', 'error', 'critical'], help='Logging level for log file.')
         self.add_argument('--log-file', help='File to write log messages to')
         self.add_argument('--color', action='store_boolean', default=True, help='color in output')
         self.add_argument('--config-file', help='The location for the configuration file')
@@ -396,7 +466,7 @@ class MILC(object):
         return self._entrypoint(self)
 
     def entrypoint(self, description):
-        """Set the entrypoint for when no subcommand is provided.
+        """Decorator that marks the entrypoint for simple scripts without subcommands.
         """
         if self._inside_context_manager:
             raise RuntimeError('You must run this before cli()!')
@@ -472,18 +542,17 @@ class MILC(object):
 
         self.acquire_lock()
 
-        if self.config['general']['verbose']:
+        if self.config.general.verbose:
             self.log_print_level = logging.DEBUG
 
-        self.log_file = self.config['general']['log_file'] or self.log_file
-        self.log_file_format = self.config['general']['log_file_fmt']
-        self.log_file_format = ANSIStrippingFormatter(self.config['general']['log_file_fmt'], self.config['general']['datetime_fmt'])
-        self.log_format = self.config['general']['log_fmt']
+        self.log_file = self.config.general.log_file or self.log_file
+        self.log_file_format = ANSIStrippingFormatter(self.config.general.log_file_fmt, self.config.general.datetime_fmt)
+        self.log_file_level = getattr(logging, self.config.general.log_file_level.upper())
 
         if self.config.general.color:
-            self.log_format = ANSIEmojiLoglevelFormatter(self.args.log_fmt, self.config.general.datetime_fmt)
+            self.log_format = ANSIEmojiLoglevelFormatter(self.config.general.log_fmt, self.config.general.datetime_fmt)
         else:
-            self.log_format = ANSIStrippingFormatter(self.args.log_fmt, self.config.general.datetime_fmt)
+            self.log_format = ANSIStrippingEmojiLoglevelFormatter(self.config.general.log_fmt, self.config.general.datetime_fmt)
 
         if self.log_file:
             self.log_file_handler = logging.FileHandler(self.log_file, self.log_file_mode)
